program txt2pdf2;

{$mode objfpc}{$H+}

uses
  Classes, SysUtils, CustApp, fpreport, fpreportpdfexport, fpTTF,
  fpreportstreamer, fpjson, jsonparser;

type

  { TPrintApplication }

  TPrintApplication = class(TCustomApplication)
  private
    FReport : TFPReport;
    FLines : TStringList;
    FData : TFPReportUserData;
    FLineIndex : Integer;
    procedure DoFirst(Sender: TObject);
    procedure DoGetEOF(Sender: TObject; var IsEOF: boolean);
    procedure DoGetNames(Sender: TObject; List: TStrings);
    procedure DoGetNext(Sender: TObject);
    procedure DoGetValue(Sender: TObject; const AValueName: string; var AValue: variant);
    procedure LoadReportDesign;
    Procedure SaveReportDesign;
  protected
    procedure DoRun; override;
  public
    Constructor Create(AOwner : TComponent); override;
    Destructor destroy; override;
  end;

{ TPrintApplication }

procedure TPrintApplication.DoGetNames(Sender: TObject; List: TStrings);
begin
  List.Add('Line');
end;

procedure TPrintApplication.DoGetEOF(Sender: TObject; var IsEOF: boolean);
begin
  isEOF:=FLineIndex>=FLines.Count;
end;

procedure TPrintApplication.DoFirst(Sender: TObject);
begin
  FLineIndex:=0;
end;

procedure TPrintApplication.DoGetNext(Sender: TObject);
begin
  Inc(FLineIndex);
end;

procedure TPrintApplication.DoGetValue(Sender: TObject; const AValueName: string; var AValue: variant);
begin
  Avalue:=FLines[FLineIndex];
end;

procedure TPrintApplication.SaveReportDesign;

Var
  J : TFPReportJSONStreamer;
  F : TFileStream;
  S : TJSONStringType;

begin
  F:=Nil;
  J:=TFPReportJSONStreamer.Create(Self);
  try
    FReport.WriteElement(J);
    F:=TFileStream.Create('txt2pdf.fpr',fmCreate);
    S:=J.JSON.FormatJSON();
    F.WriteBuffer(S[1],Length(S));
  finally
    F.Free;
    J.Free;
  end;
end;

procedure TPrintApplication.LoadReportDesign;

Var
  J : TFPReportJSONStreamer;
  F : TFileStream;
  O : TJSONObject;

begin
  J:=Nil;
  F:=TFileStream.Create('txt2pdf.fpr',fmOpenRead);
  try
    O:=GetJSON(F) as TJSONObject;
    J:=TFPReportJSONStreamer.Create(Self);
    J.JSON:=O;
    J.OwnsJSON:=True;
    FReport.ReadElement(J);
  finally
    F.Free;
    J.Free;
  end;
end;

procedure TPrintApplication.DoRun;

Var
  PDF : TFPReportExportPDF;

begin
  Terminate;
  FLines.LoadFromFile(ParamStr(1));
  gTTFontCache.ReadStandardFonts;
  gTTFontCache.BuildFontCache;
  PaperManager.RegisterStandardSizes;
  // Load report design from file
  LoadReportDesign;
  // Set up data
  FData.OnGetNames:=@DoGetNames;
  FData.OnNext:=@DoGetNext;
  FData.OnGetValue:=@DoGetValue;
  FData.OnGetEOF:=@DoGetEOF;
  FData.OnFirst:=@DoFirst;
  // Go !
  FReport.RunReport;
  PDF:=TFPReportExportPDF.Create(Self);
  try
    PDF.FileName:=ChangeFileExt(Paramstr(1),'.pdf');
    FReport.RenderReport(PDF);
  finally
    PDF.Free;
  end;
end;

constructor TPrintApplication.Create(AOwner: TComponent);

begin
  Inherited;
  FReport:=TFPReport.Create(Self);
  FLines:=TStringList.Create;
  FData:=TFPReportUserData.Create(Self);
  FData.Name:='Data';
  Freport.ReportData.AddReportData(FData);
end;

destructor TPrintApplication.destroy;
begin
  FreeAndNil(FData);
  FreeAndNil(FLines);
  FreeAndNil(FReport);
  inherited destroy;
end;

var
  Application: TPrintApplication;
begin
  Application:=TPrintApplication.Create(nil);
  Application.Title:='Print File Application';
  Application.Run;
  Application.Free;
end.

